__author__ = 'labx'

"""
=========
Tools
=========

Widgets for common tools.

"""

# Category description for the widget registry

NAME = "Tools"

DESCRIPTION = "Widgets for Common Tools."

BACKGROUND = "#FAAC58"

ICON = "icons/tools.png"

PRIORITY = 1
