
# THIS FILE IS GENERATED FROM OASYS SETUP.PY
short_version = '1.1.31'
version = '1.1.31'
full_version = '1.1.31'
git_revision = '14d2224b9b2948a18b73968940155038ccf1b59e'
release = True

if not release:
    version = full_version
    short_version += ".dev"
